//-----------------------------------------------------------------------------
// Credits
//-----------------------------------------------------------------------------
// Original by sprinkles, modified by Le Codex

//-----------------------------------------------------------------------------
// Constants and netprops folding
//-----------------------------------------------------------------------------
::ROOT <- getroottable();
if (!("ConstantNamingConvention" in ROOT)) // make sure folding is only done once
{
	foreach (a,b in Constants)
		foreach (k,v in b)
			ROOT[k] <- v != null ? v : 0;
    
    foreach (k, v in ::NetProps.getclass())
        if (k != "IsValid")
            ROOT[k] <- ::NetProps[k].bindenv(::NetProps)
}

//-----------------------------------------------------------------------------
// Variables
//-----------------------------------------------------------------------------

::m_iMaxPlayers <- MaxClients().tointeger();

//-----------------------------------------------------------------------------
// Functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Purpose: precache a particle
// Input  : string
//-----------------------------------------------------------------------------
::PrecacheParticle <- function(name)
{
    PrecacheEntityFromTable({ classname = "info_particle_system", effect_name = name })
}

//-----------------------------------------------------------------------------
// Purpose: Get two vectors and return the distance
// Input  : vectors
// Output : Returns Length of the two vectors.
//-----------------------------------------------------------------------------
::Distance3D <- function(v1,v2)
{
	local a = (v2.x-v1.x);
	local b = (v2.y-v1.y);
	local c = (v2.z-v1.z);

	return sqrt((a*a)+(b*b)+(c*c));
}

//-----------------------------------------------------------------------------
// Purpose: Gives the closest alive player to a given point and with a possible ignore
// Input : Vector, number, handle?
// Output : Returns the closest alive player
//-----------------------------------------------------------------------------
::FindNearestAlivePlayer <- function(point, maxDistance = 2048, ignore = null)
{
    // Initialize the return values
    local nearestPlayer = null
    local minDistance = maxDistance

    // For each player in the max range...
    for (local player; player= Entities.FindByClassnameWithin(player, "player", point, maxDistance); ) {
        // If that player is not alive, skip
        if (!player.IsAlive()) continue
        // If it's the ignored player, skip
        if (player == ignore) continue
        
        // Check for distance
        local distance = Distance3D(player.GetOrigin(), point);
        // If it's closer than our curent minDistance, then set that player as the closest one and update minDistance
        if (distance <= minDistance && distance > 0) {
            minDistance = distance
            nearestPlayer = player
        }
    }

    // Returns the closest player
    return nearestPlayer
}

//-----------------------------------------------------------------------------
// Purpose: Check the dev level
// Output : Returns true on success, false on failure.
//-----------------------------------------------------------------------------
::IsInDebugMode <- function()
{
	return GetDeveloperLevel() > 0
}

//-----------------------------------------------------------------------------
// Purpose: Check the dev level and print text based on it
// input  : pMsg.
//-----------------------------------------------------------------------------
::DebugPrint <- function(pMsg)
{
	if(IsInDebugMode())
    {
        print(pMsg)
    }
}

//-----------------------------------------------------------------------------
// Purpose: Check the dev level and print text based on it
// input  : pMsg.
//-----------------------------------------------------------------------------
::DebugPrintl <- function(pMsg)
{
	if(IsInDebugMode())
    {
        printl(pMsg)
    }
}

//-----------------------------------------------------------------------------
// Purpose: Check the dev level and print text based on it
// input  : pMsg.
//-----------------------------------------------------------------------------
::DebugError <- function(pMsg)
{
	if(IsInDebugMode())
    {
        error(pMsg)
    }
}

//-----------------------------------------------------------------------------
// Purpose: split a array into seperate things and print it
// input  : array.
//-----------------------------------------------------------------------------
::arraysplit <-  function(array, int = 0)
{
    local arrayamount = array.len() - 1
    if (int != 0)
    {
        if (arrayamount == int)
        {
            return array[int]
        }
    }
    else
    {
        for (local i = 0; i <= arrayamount ; i++)
        {
            return array[i]
        }
    }
}

//-----------------------------------------------------------------------------
// Purpose: Adds or remove score based on events
// input  : m_iScore, player
//-----------------------------------------------------------------------------
::CTFPlayer.ScorePoints <- function(m_iScore)
{
	local player = this;
	local is_bot = player.IsFakeClient()
    if (!is_bot) player.AddFlag(Constants.FPlayer.FL_FAKECLIENT)

    SendGlobalGameEvent("player_escort_score",
    {
        player = player.entindex(),
        points = m_iScore
    })

    if (!is_bot) player.RemoveFlag(Constants.FPlayer.FL_FAKECLIENT)
}

::CTFBot.ScorePoints <- CTFPlayer.ScorePoints

//-----------------------------------------------------------------------------
// Purpose: Returns the players name ie "sprinkles" or "Seacat08"
// input  : player.
//-----------------------------------------------------------------------------
::CTFPlayer.GetPlayerName <- function()
{
	local player = this;
    return NetProps.GetPropString(player, "m_szNetname");
}

::CTFBot.GetPlayerName <- CTFPlayer.GetPlayerName


//-----------------------------------------------------------------------------
// Purpose: delays a given function by a set time
// input  : functionname, delay.
//-----------------------------------------------------------------------------
::CTFPlayer.SetDelayedFunction <-  function(functionname, delay)
{
	local player = this;
    EntFireByHandle(player, "RunScriptCode", functionname, delay, player, player)
}

::CTFBot.SetDelayedFunction <- CTFPlayer.SetDelayedFunction

//-----------------------------------------------------------------------------
// Purpose: Return the players Steamid for dev reasons
// input  : player.
//-----------------------------------------------------------------------------
::CTFPlayer.GetPlayerSteamID <- function()
{
	local player = this;
    return NetProps.GetPropString(player, "m_szNetworkIDString");
}

::CTFBot.GetPlayerSteamID <- CTFPlayer.GetPlayerSteamID